@@ -17,9 +17,8 @@ from django.core.urlresolvers import get_mod_func
 from django.utils.encoding import python_2_unicode_compatible
 from django.utils.translation import ugettext as _
 
-from djblets.deprecation import RemovedInDjblets20Warning
 from djblets.extensions.errors import InstallExtensionMediaError
-from djblets.extensions.settings import Settings
+from djblets.extensions.settings import ExtensionSettings
 from djblets.util.decorators import cached_property
 
 
@@ -140,7 +139,7 @@ class Extension(object):
             The list of Django middleware instances. Each will be an instance
             of a class listed in :py:attr:`middleware`.
 
-        settings (djblets.extensions.settings.Settings):
+        settings (djblets.extensions.settings.ExtensionSettings):
             The settings for the extension.
     """
 
@@ -269,7 +268,7 @@ class Extension(object):
         """
         self.extension_manager = extension_manager
         self.hooks = set()
-        self.settings = Settings(self)
+        self.settings = ExtensionSettings(self)
         self.admin_site = None
         self.middleware_instances = []
 
@@ -427,11 +426,11 @@ class ExtensionInfo(object):
 
         try:
             # Wheel, or other modern package.
-            lines = dist.get_metadata_lines('METADATA')
+            lines = list(dist.get_metadata_lines('METADATA'))
         except IOError:
             try:
                 # Egg, or other legacy package.
-                lines = dist.get_metadata_lines('PKG-INFO')
+                lines = list(dist.get_metadata_lines('PKG-INFO'))
             except IOError:
                 lines = []
                 logger.error('No METADATA or PKG-INFO found for the package '
@@ -439,21 +438,26 @@ class ExtensionInfo(object):
                              'the extension may be missing.',
                              extension_id)
 
-        # Both Python 2 and 3 will use byte strings here.
-        data = b'\n'.join(lines)
+        # pkg_resources on Python 3 will always give us back Unicode strings,
+        # but Python 2 may give us back either Unicode or byte strings.
+        if lines and isinstance(lines[0], bytes):
+            data = b'\n'.join(lines)
 
-        # Try to decode the PKG-INFO content. If no decoding method is
-        # successful then the PKG-INFO content will remain unchanged and
-        # processing will continue with the parsing.
-        for enc in cls.encodings:
-            try:
-                data = data.decode(enc)
-                break
-            except UnicodeDecodeError:
-                continue
+            # Try to decode the PKG-INFO content. If no decoding method is
+            # successful then the PKG-INFO content will remain unchanged and
+            # processing will continue with the parsing.
+            for enc in cls.encodings:
+                try:
+                    data = data.decode(enc)
+                    break
+                except UnicodeDecodeError:
+                    continue
+            else:
+                logger.warning('Failed decoding PKG-INFO content for '
+                               'extension %s',
+                               entrypoint.name)
         else:
-            logger.warning('Failed decoding PKG-INFO content for extension %s',
-                           entrypoint.name)
+            data = '\n'.join(lines)
 
         p = FeedParser()
         p.feed(data)
@@ -488,36 +492,13 @@ class ExtensionInfo(object):
         try:
             issubclass(ext_class, Extension)
         except TypeError:
-            try:
-                is_entrypoint = (hasattr(ext_class, 'dist') and
-                                 issubclass(package_name, Extension))
-            except TypeError:
-                is_entrypoint = False
+            logger.error('Unexpected parameters passed to '
+                         'ExtensionInfo.__init__: ext_class=%r, '
+                         'package_name=%r, metadata=%r',
+                         ext_class, package_name, metadata)
 
-            if is_entrypoint:
-                # These are really (probably) an entrypoint and class,
-                # respectively. Fix up the variables.
-                entrypoint, ext_class = ext_class, package_name
-
-                metadata = self._get_metadata_from_entrypoint(entrypoint,
-                                                              ext_class.id)
-                package_name = metadata.get('Name')
-
-                # Warn after the above. Something about the above calls cause
-                # warnings to be reset.
-                warnings.warn(
-                    'ExtensionInfo.__init__() no longer accepts an '
-                    'EntryPoint. Please update your code to call '
-                    'ExtensionInfo.create_from_entrypoint() instead.',
-                    RemovedInDjblets20Warning)
-            else:
-                logger.error('Unexpected parameters passed to '
-                             'ExtensionInfo.__init__: ext_class=%r, '
-                             'package_name=%r, metadata=%r',
-                             ext_class, package_name, metadata)
-
-                raise TypeError(
-                    _('Invalid parameters passed to ExtensionInfo.__init__'))
+            raise TypeError(
+                _('Invalid parameters passed to ExtensionInfo.__init__'))
 
         # Set the base information from the extension and the package.
         self.package_name = package_name
@@ -534,6 +515,8 @@ class ExtensionInfo(object):
         self.enabled = False
         self.installed = False
         self.requirements = []
+        self.apps_registered = False
+        self.context_processors_registered = False
 
         # Set information from the provided metadata.
         if ext_class.metadata is not None:
