@@ -13,6 +13,7 @@ import tempfile
 import threading
 import time
 import traceback
+import warnings
 import weakref
 from contextlib import contextmanager
 from importlib import import_module
@@ -20,22 +21,19 @@ from importlib import import_module
 from django.conf import settings
 from django.conf.urls import include, url
 from django.contrib.admin.sites import AdminSite
-from django.core.management import call_command
-from django.core.management.base import CommandError
 from django.core.urlresolvers import reverse
 from django.db import IntegrityError
-from django.db.utils import DEFAULT_DB_ALIAS
 from django.utils import six
 from django.utils.module_loading import module_has_submodule
-from django.utils.six.moves import cStringIO as StringIO
+from django.utils.six.moves import reload_module
 from django.utils.translation import ugettext as _
 from pipeline.conf import settings as pipeline_settings
 from setuptools.command import easy_install
 
 try:
-    from django_evolution.evolve import get_unapplied_evolutions
+    from django_evolution.evolve import Evolver
 except ImportError:
-    get_unapplied_evolutions = None
+    Evolver = None
 
 try:
     # Django >= 1.7
@@ -286,6 +284,7 @@ class ExtensionManager(object):
         self._block_sync_gen = False
 
         self.dynamic_urls = DynamicURLResolver()
+        self._extension_list_url = None
 
         # Extension middleware instances, ordered by dependencies.
         self.middleware = []
@@ -346,14 +345,20 @@ class ExtensionManager(object):
     def get_absolute_url(self):
         """Return an absolute URL to the view for listing extensions.
 
-        By default, this simply looks up the "extension-list" URL. Subclasses
-        can override this to provide a more specific URL.
+        By default, this simply looks up the "extension-list" URL.
+
+        Subclasses can override this to provide a more specific URL, but should
+        take care to cache the result in order to avoid unwanted lookups caused
+        by URL resolver cache flushes.
 
         Returns:
             unicode:
             The URL to the extension list view.
         """
-        return reverse('extension-list')
+        if not self._extension_list_url:
+            self._extension_list_url = reverse('extension-list')
+
+        return self._extension_list_url
 
     def get_can_disable_extension(self, registered_extension):
         """Return whether an extension can be disabled.
@@ -900,9 +905,14 @@ class ExtensionManager(object):
 
             extension.info.installed = extension.registration.installed
             extension.info.enabled = True
+
             new_installed_apps = self._add_to_installed_apps(extension)
+            extension.info.apps_registered = True
+
             self._context_processors_setting.add_list(
                 extension.context_processors)
+            extension.info.context_processors_registered = True
+
             clear_template_tag_caches()
             ext_class.instance = extension
 
@@ -944,14 +954,15 @@ class ExtensionManager(object):
                 ext_class.registration.installed = True
                 ext_class.registration.save(update_fields=('installed',))
         except EnablingExtensionError as e:
-            # Raise this as-is.
-            del self._extension_instances[extension_id]
+            self._uninit_extension(extension)
 
             logger.exception('Error initializing extension %s: %s',
                              ext_class.id, e)
+
+            # Raise this as-is.
             raise
         except Exception as e:
-            del self._extension_instances[extension_id]
+            self._uninit_extension(extension)
 
             logger.exception('Unexpected error initializing extension %s: %s',
                              ext_class.id, e)
@@ -970,6 +981,9 @@ class ExtensionManager(object):
         This will create any database tables that need to be created and
         the perform a database migration, if needed.
 
+        This requires that Django Evolution be installed. Otherwise, models
+        will not be synced to the database.
+
         Args:
             ext_class (type):
                 The extension class owning the database models.
@@ -977,6 +991,11 @@ class ExtensionManager(object):
             new_installed_app_names (list of unicode):
                 The list of new Django app names that are installed by an
                 extension.
+
+        Raises:
+            djblets.extensions.errors.InstallExtensionError:
+                The extension's database entries could not be installed to
+                the database or upgraded.
         """
         installed_apps = self._get_app_modules_with_models(
             new_installed_app_names)
@@ -984,16 +1003,27 @@ class ExtensionManager(object):
         if not installed_apps:
             return
 
-        if apps:
-            # Django >= 1.7
-            call_command('migrate', run_syncdb=True, verbosity=0,
-                         interactive=False)
-        else:
-            # Django == 1.6
-            call_command('syncdb', verbosity=0, interactive=False)
+        if not Evolver:
+            warnings.warn('Extension database models will not be synced '
+                          'to the database. Django Evolution must be '
+                          'installed.')
+            return
 
-        # Run evolve to do any table modification.
-        self._migrate_extension_models(ext_class, installed_apps)
+        evolver = Evolver()
+
+        for app in installed_apps:
+            evolver.queue_evolve_app(app)
+
+        if evolver.get_evolution_required():
+            try:
+                evolver.evolve()
+            except Exception as e:
+                logger.exception('Error evolving extension models for %s: %s',
+                                 ext_class.id, e)
+
+                raise InstallExtensionError(
+                    six.text_type(e),
+                    self._store_load_error(ext_class.id, e))
 
     def _get_app_modules_with_models(self, app_names):
         """Return app modules containing models for each app name.
@@ -1049,25 +1079,53 @@ class ExtensionManager(object):
         """
         try:
             extension.shutdown()
+        except Exception as e:
+            logger.exception('Unexpected error shutting down extension %s: %s',
+                             extension.id, e)
 
-            if hasattr(extension, 'admin_urlpatterns'):
-                self.dynamic_urls.remove_patterns(
-                    extension.admin_urlpatterns)
+        if getattr(extension, 'admin_urlpatterns', []):
+            try:
+                self.dynamic_urls.remove_patterns(extension.admin_urlpatterns)
+            except Exception as e:
+                logger.exception('Unexpected error removing custom admin URL '
+                                 'patterns for extension %s: %s',
+                                 extension.id, e)
 
-            if hasattr(extension, 'admin_site_urlpatterns'):
+        if getattr(extension, 'admin_site_urlpatterns', []):
+            try:
                 self.dynamic_urls.remove_patterns(
                     extension.admin_site_urlpatterns)
+            except Exception as e:
+                logger.exception('Unexpected error removing AdminSite URL '
+                                 'patterns for extension %s: %s',
+                                 extension.id, e)
 
-            if hasattr(extension, 'admin_site'):
-                del extension.admin_site
+        if hasattr(extension, 'admin_site'):
+            del extension.admin_site
 
-            self._context_processors_setting.remove_list(
-                extension.context_processors)
-            self._remove_from_installed_apps(extension)
+        if (extension.info.context_processors_registered and
+            extension.context_processors):
+            try:
+                self._context_processors_setting.remove_list(
+                    extension.context_processors)
+            except Exception as e:
+                logger.exception('Unexpected error unregistering context '
+                                 'processors for extension %s: %s',
+                                 extension.id, e)
+
+        if extension.info.apps_registered:
+            try:
+                self._remove_from_installed_apps(extension)
+            except Exception as e:
+                logger.exception('Unexpected error removing installed app '
+                                 'modules for extension %s: %s',
+                                 extension.id, e)
+
+        try:
             clear_template_tag_caches()
         except Exception as e:
-            logger.exception('Unexpected error uninitializing extension %s: '
-                             '%s',
+            logger.exception('Unexpected error clearing template tag caches '
+                             'when uninitializing extension %s: %s',
                              extension.id, e)
 
         extension.info.enabled = False
@@ -1436,7 +1494,7 @@ class ExtensionManager(object):
                 # If the extension has been loaded previously and we are
                 # re-enabling it, we must reload the module. Just importing
                 # again will not cause the ModelAdmins to be registered.
-                reload(sys.modules[admin_module_name])
+                reload_module(sys.modules[admin_module_name])
             else:
                 import_module(admin_module_name)
         except ImportError:
@@ -1577,62 +1635,6 @@ class ExtensionManager(object):
         self._gen_sync.mark_updated()
         settings.AJAX_SERIAL = self._gen_sync.sync_gen
 
-    def _migrate_extension_models(self, ext_class, installed_apps):
-        """Perform database migrations for an extension's models.
-
-        This will call out to Django Evolution to handle the migrations.
-
-        Args:
-            ext_class (djblets.extensions.extension.Extension):
-                The class for the extension to migrate.
-
-            installed_apps (list of module):
-                The list of app modules (technically :file:`models.py`-like
-                modules -- this is a legacy Django definition) to migrate.
-        """
-        if get_unapplied_evolutions is None:
-            # Django Evolution isn't installed. Extensions with evolutions
-            # are not supported.
-            return
-
-        needs_evolution = any(
-            get_unapplied_evolutions(installed_app, DEFAULT_DB_ALIAS)
-            for installed_app in installed_apps
-        )
-
-        if not needs_evolution:
-            # This extension either doesn't use evolutions, or doesn't have
-            # any that need to be applied.
-            return
-
-        try:
-            stream = StringIO()
-            call_command('evolve',
-                         verbosity=0,
-                         interactive=False,
-                         execute=True,
-                         stdout=stream,
-                         stderr=stream)
-            output = stream.getvalue()
-
-            if output:
-                logger.info('Evolved extension models for %s: %s',
-                            ext_class.id, stream.read())
-
-            stream.close()
-        except CommandError as e:
-            # Something went wrong while running django-evolution, so
-            # grab the output.  We can't raise right away because we
-            # still need to put stdout back the way it was
-            output = stream.getvalue()
-            stream.close()
-
-            logger.exception('Error evolving extension models: %s: %s',
-                             e, output)
-
-            load_error = self._store_load_error(ext_class.id, output)
-            raise InstallExtensionError(six.text_type(e), load_error)
-
     def _recalculate_middleware(self):
         """Recalculate the list of middleware.
 
