@@ -10,14 +10,11 @@ import tempfile
 import threading
 import time
 
-import django
-import nose
 from django.conf import settings
-from django.core.management import call_command
 from django.db import connection
 from django.template import Context, Template, TemplateSyntaxError
 from django.test.utils import override_settings
-from django.utils.six.moves import cPickle as pickle
+from django_evolution.evolve import Evolver
 from kgb import SpyAgency
 from pipeline.conf import settings as pipeline_settings
 
@@ -28,11 +25,6 @@ except ImportError:
     # Django >= 1.9
     get_templatetags_modules = None
 
-try:
-    import django_evolution
-except ImportError:
-    django_evolution = None
-
 from djblets.extensions.errors import EnablingExtensionError
 from djblets.extensions.extension import Extension
 from djblets.extensions.hooks import URLHook
@@ -57,9 +49,19 @@ class TestExtension(Extension):
     ]
 
 
+class HooksTestExtension(TestExtension):
+    def initialize(self):
+        self.url_hook = URLHook(self, ())
+
+
 class ExtensionManagerTests(SpyAgency, ExtensionTestsMixin, TestCase):
     """Unit tests for djblets.extensions.manager.ExtensionManager."""
 
+    def tearDown(self):
+        super(ExtensionManagerTests, self).tearDown()
+
+        self.assertEqual(len(URLHook.hooks), 0)
+
     def test_added_to_extension_managers(self):
         """Testing ExtensionManager registration"""
         self.manager = TestExtensionManager([], '')
@@ -91,6 +93,9 @@ class ExtensionManagerTests(SpyAgency, ExtensionTestsMixin, TestCase):
 
     def test_load_full_reload_hooks(self):
         """Testing ExtensionManager.load with full_reload=True"""
+        # NOTE: We're not using HooksTestExtension here, because we want to
+        #       manually set up a hook and not have to return once we
+        #       perform a reload.
         extension = self.setup_extension(TestExtension)
 
         self.assertEqual(len(self.manager.get_installed_extensions()), 1)
@@ -167,6 +172,71 @@ class ExtensionManagerTests(SpyAgency, ExtensionTestsMixin, TestCase):
                          self.test_project_name)
         self.assertTrue(TestExtension.registration.enabled)
 
+    def test_load_with_admin_site_failure(self):
+        """Testing ExtensionManager.load with extension error setting up
+        admin site
+        """
+        class AdminSiteTestExtension(HooksTestExtension):
+            has_admin_site = True
+
+        def _init_admin_site(*args, **kwargs):
+            raise Exception('admin site failed!')
+
+        extension = self.setup_extension(AdminSiteTestExtension)
+        manager = self.manager
+
+        self.assertEqual(manager.get_installed_extensions(),
+                         [AdminSiteTestExtension])
+        self.assertEqual(manager.get_enabled_extensions(), [extension])
+        self.assertIs(AdminSiteTestExtension.instance, extension)
+        self.assertEqual(len(URLHook.hooks), 1)
+
+        self.spy_on(TestExtensionManager._init_admin_site,
+                    owner=TestExtensionManager,
+                    call_fake=_init_admin_site)
+
+        self.manager.load(full_reload=True)
+
+        self.assertEqual(manager.get_installed_extensions(),
+                         [AdminSiteTestExtension])
+        self.assertEqual(manager.get_enabled_extensions(), [])
+        self.assertIsNone(AdminSiteTestExtension.instance)
+        self.assertEqual(len(URLHook.hooks), 0)
+        self.assertTrue(manager._load_errors[extension.id].startswith(
+            'admin site failed!\n\nTraceback'))
+
+    def test_load_with_admin_urls_failure(self):
+        """Testing ExtensionManager.load with extension error setting up
+        admin URLs
+        """
+        def _install_admin_urls(*args, **kwargs):
+            raise Exception('admin URLs failed!')
+
+        extension = self.setup_extension(HooksTestExtension)
+        manager = self.manager
+
+        self.assertEqual(manager.get_installed_extensions(),
+                         [HooksTestExtension])
+        self.assertEqual(manager.get_enabled_extensions(), [extension])
+        self.assertIs(HooksTestExtension.instance, extension)
+        self.assertEqual(len(URLHook.hooks), 1)
+        self.assertEqual(manager._load_errors, {})
+
+        self.spy_on(TestExtensionManager._install_admin_urls,
+                    owner=TestExtensionManager,
+                    call_fake=_install_admin_urls)
+
+        self.manager.load(full_reload=True)
+
+        self.assertEqual(manager.get_installed_extensions(),
+                         [HooksTestExtension])
+        self.assertEqual(manager.get_enabled_extensions(), [])
+        self.assertIsNone(HooksTestExtension.instance)
+        self.assertEqual(len(URLHook.hooks), 0)
+        self.assertIn(extension.id, manager._load_errors)
+        self.assertTrue(manager._load_errors[extension.id].startswith(
+            'admin URLs failed!\n\nTraceback'))
+
     def test_sync_database_with_no_settings_version(self):
         """Testing ExtensionManager synchronizes database when no version
         found in settings (new install)
@@ -367,17 +437,22 @@ class ExtensionManagerTests(SpyAgency, ExtensionTestsMixin, TestCase):
         """Testing ExtensionManager media installation with permission error
         when copying media files
         """
-        extension = self.setup_extension(TestExtension, enable=False)
-        TestExtension.instance = extension
+        self.assertEqual(len(URLHook.hooks), 0)
+        extension = self.setup_extension(HooksTestExtension, enable=False)
+        extension.shutdown()
+
+        manager = self.manager
+
+        self.assertEqual(len(URLHook.hooks), 0)
 
         self._rebuild_media_dirs(extension)
 
-        self.manager.should_install_static_media = True
+        manager.should_install_static_media = True
 
         def _copytree(*args, **kwargs):
             raise shutil.Error()
 
-        self.spy_on(self.manager.install_extension_media)
+        self.spy_on(manager.install_extension_media)
         self.spy_on(extension.info.has_resource,
                     call_fake=lambda *args, **kwargs: True)
         self.spy_on(shutil.copytree, call_fake=_copytree)
@@ -389,36 +464,44 @@ class ExtensionManagerTests(SpyAgency, ExtensionTestsMixin, TestCase):
             'are owned by the web server.'
         ) % os.path.join(
             settings.STATIC_ROOT, 'ext',
-            'djblets.extensions.tests.test_extension_manager.TestExtension'
+            'djblets.extensions.tests.test_extension_manager.'
+            'HooksTestExtension'
         )
 
         with self.assertRaisesMessage(EnablingExtensionError, message):
-            self.manager.enable_extension(TestExtension.id)
+            manager.enable_extension(HooksTestExtension.id)
 
         self.assertFalse(extension.registration.installed)
-        self.assertEqual(len(self.manager.install_extension_media.calls), 1)
+        self.assertEqual(len(manager.install_extension_media.calls), 1)
         self.assertFalse(os.path.exists(
             os.path.join(extension.info.installed_static_path, '.version')))
+        self.assertEqual(manager.get_installed_extensions(),
+                         [HooksTestExtension])
+        self.assertEqual(manager.get_enabled_extensions(), [])
+        self.assertIsNone(HooksTestExtension.instance)
+        self.assertEqual(len(URLHook.hooks), 0)
 
     def test_install_extension_media_with_lock_error(self):
         """Testing ExtensionManager media installation with error when locking
         static directory
         """
-        extension = self.setup_extension(TestExtension, enable=False)
-        TestExtension.instance = extension
+        extension = self.setup_extension(HooksTestExtension, enable=False)
+        extension.shutdown()
+
+        manager = self.manager
 
         self._rebuild_media_dirs(extension)
 
-        self.manager.should_install_static_media = True
-        self.manager._MEDIA_LOCK_SLEEP_TIME_SECS = 0
+        manager.should_install_static_media = True
+        manager._MEDIA_LOCK_SLEEP_TIME_SECS = 0
 
         def _open_lock_file(*args, **kwargs):
             raise IOError(errno.EAGAIN, 'Try again')
 
-        self.spy_on(self.manager.install_extension_media)
-        self.spy_on(self.manager._install_extension_media_internal,
+        self.spy_on(manager.install_extension_media)
+        self.spy_on(manager._install_extension_media_internal,
                     call_original=False)
-        self.spy_on(self.manager._open_lock_file,
+        self.spy_on(manager._open_lock_file,
                     call_fake=_open_lock_file)
 
         message = (
@@ -429,22 +512,29 @@ class ExtensionManagerTests(SpyAgency, ExtensionTestsMixin, TestCase):
         ) % (
             os.path.join(
                 settings.STATIC_ROOT, 'ext',
-                'djblets.extensions.tests.test_extension_manager.TestExtension'
+                'djblets.extensions.tests.test_extension_manager'
+                '.HooksTestExtension'
             ),
             tempfile.gettempdir()
         )
 
         with self.assertRaisesMessage(EnablingExtensionError, message):
-            self.manager.enable_extension(TestExtension.id)
+            manager.enable_extension(HooksTestExtension.id)
 
         self.assertFalse(extension.registration.installed)
 
-        self.assertEqual(len(self.manager.install_extension_media.calls), 1)
-        self.assertFalse(self.manager._install_extension_media_internal.called)
+        self.assertEqual(len(manager.install_extension_media.calls), 1)
+        self.assertFalse(manager._install_extension_media_internal.called)
 
         self.assertFalse(os.path.exists(
             os.path.join(extension.info.installed_static_path, '.version')))
 
+        self.assertEqual(manager.get_installed_extensions(),
+                         [HooksTestExtension])
+        self.assertEqual(manager.get_enabled_extensions(), [])
+        self.assertIsNone(HooksTestExtension.instance)
+        self.assertEqual(len(URLHook.hooks), 0)
+
     def test_install_media_concurrent_threads(self):
         """Testing ExtensionManager installs media safely with multiple
         threads
@@ -666,11 +756,12 @@ class ExtensionManagerTests(SpyAgency, ExtensionTestsMixin, TestCase):
         Template('{% load djblets_js djblets_extensions %}').render(
             Context({}))
 
-    def test_enable_extension_syncdb_with_models(self):
+    def test_enable_extension_evolve_with_models(self):
         """Testing ExtensionManager.enable_extension synchronizes database
         models
         """
-        self.spy_on(call_command)
+        self.spy_on(Evolver.evolve,
+                    owner=Evolver)
 
         class TestExtensionWithApps(Extension):
             apps = [
@@ -679,14 +770,7 @@ class ExtensionManagerTests(SpyAgency, ExtensionTestsMixin, TestCase):
 
         extension = self.setup_extension(TestExtensionWithApps)
 
-        # There should be a call to call_command for either syncdb or
-        # migrate (depending on Django version). Note that syncdb at least
-        # will perform its own calls, so we don't want to expect a certain
-        # number of calls.
-        if django.VERSION[:2] >= (1, 7):
-            self.assertTrue(call_command.called_with('migrate'))
-        else:
-            self.assertTrue(call_command.called_with('syncdb'))
+        self.assertTrue(Evolver.evolve.called)
 
         from djblets.extensions.tests.apps.model_tests.models import \
             TestExtensionModel
@@ -700,11 +784,12 @@ class ExtensionManagerTests(SpyAgency, ExtensionTestsMixin, TestCase):
         self.manager.enable_extension(extension.id)
         self.assertEqual(TestExtensionModel.objects.count(), 1)
 
-    def test_enable_extension_syncdb_without_models(self):
+    def test_enable_extension_evolve_without_models(self):
         """Testing ExtensionManager.enable_extension does not synchronize
         database models if extension does not have any
         """
-        self.spy_on(call_command)
+        self.spy_on(Evolver.evolve,
+                    owner=Evolver)
 
         class TestExtensionWithApps(Extension):
             apps = [
@@ -713,19 +798,17 @@ class ExtensionManagerTests(SpyAgency, ExtensionTestsMixin, TestCase):
 
         self.setup_extension(TestExtensionWithApps)
 
-        self.assertFalse(call_command.called)
+        self.assertFalse(Evolver.evolve.called)
 
     def test_enable_extension_evolve_with_pending_evolutions(self):
         """Testing ExtensionManager.enable_extension evolves database models
         when pending evolutions found
         """
-        if django_evolution is None:
-            raise nose.SkipTest()
-
         from django_evolution.models import Version
-        from django_evolution.signature import create_model_sig
+        from django_evolution.signature import AppSignature, ModelSignature
 
-        self.spy_on(call_command)
+        self.spy_on(Evolver.evolve,
+                    owner=Evolver)
 
         class TestExtensionWithApps(Extension):
             apps = [
@@ -735,33 +818,29 @@ class ExtensionManagerTests(SpyAgency, ExtensionTestsMixin, TestCase):
         # We need to set some initial state in the database for the model and
         # for the evolution history.
         connection.cursor().execute(
-            'CREATE TABLE evolve_tests_testextensionmodel ('
+            'CREATE TABLE evolve_tests_testevolveextensionmodel ('
             '    id INTEGER PRIMARY KEY AUTOINCREMENT,'
             '    test_field VARCHAR(16) NOT NULL'
             ')')
 
-        from djblets.extensions.tests.apps.model_tests.models import \
-            TestExtensionModel
+        from djblets.extensions.tests.apps.evolve_tests.models import \
+            TestEvolveExtensionModel
 
         latest_version = Version.objects.current_version()
-        sig = pickle.loads(bytes(latest_version.signature))
 
-        model_sig = create_model_sig(TestExtensionModel)
-        model_sig['meta']['db_table'] = 'evolve_tests_testevolveextensionmodel'
+        model_sig = ModelSignature.from_model(TestEvolveExtensionModel)
+        model_sig.remove_field_sig('new_field')
 
-        sig['evolve_tests'] = {
-            'TestEvolveExtensionModel': model_sig,
-        }
+        app_sig = AppSignature(app_id='evolve_tests')
+        app_sig.add_model_sig(model_sig)
 
-        Version.objects.create(signature=pickle.dumps(sig))
+        latest_version.signature.add_app_sig(app_sig)
+        latest_version.save()
 
         # We can now enable the extension, which will perform an evolution.
         extension = self.setup_extension(TestExtensionWithApps)
 
-        self.assertTrue(call_command.called_with('evolve'))
-
-        from djblets.extensions.tests.apps.evolve_tests.models import \
-            TestEvolveExtensionModel
+        self.assertTrue(Evolver.evolve.called)
 
         # We should be able to create entries and query them.
         TestEvolveExtensionModel.objects.create(test_field='test')
@@ -784,13 +863,11 @@ class ExtensionManagerTests(SpyAgency, ExtensionTestsMixin, TestCase):
         """Testing ExtensionManager.enable_extension evolves database models
         when all evolutions are already applied
         """
-        if django_evolution is None:
-            raise nose.SkipTest()
-
         from django_evolution.models import Evolution, Version
-        from django_evolution.signature import create_model_sig
+        from django_evolution.signature import AppSignature, ModelSignature
 
-        self.spy_on(call_command)
+        self.spy_on(Evolver.evolve,
+                    owner=Evolver)
 
         class TestExtensionWithApps(Extension):
             apps = [
@@ -800,7 +877,7 @@ class ExtensionManagerTests(SpyAgency, ExtensionTestsMixin, TestCase):
         # We need to set some initial state in the database for the model and
         # for the evolution history.
         connection.cursor().execute(
-            'CREATE TABLE evolve_tests_testextensionmodel ('
+            'CREATE TABLE evolve_tests_testevolveextensionmodel ('
             '    id INTEGER PRIMARY KEY AUTOINCREMENT,'
             '    test_field VARCHAR(16) NOT NULL'
             ')')
@@ -809,16 +886,17 @@ class ExtensionManagerTests(SpyAgency, ExtensionTestsMixin, TestCase):
             TestExtensionModel
 
         latest_version = Version.objects.current_version()
-        sig = pickle.loads(bytes(latest_version.signature))
+        signature = latest_version.signature.clone()
 
-        model_sig = create_model_sig(TestExtensionModel)
-        model_sig['meta']['db_table'] = 'evolve_tests_testevolveextensionmodel'
+        model_sig = ModelSignature.from_model(TestExtensionModel)
+        model_sig.model_name = 'TestEvolveExtensionModel'
+        model_sig.table_name = 'evolve_tests_testevolveextensionmodel'
 
-        sig['evolve_tests'] = {
-            'TestEvolveExtensionModel': model_sig,
-        }
+        app_sig = AppSignature(app_id='evolve_tests')
+        app_sig.add_model_sig(model_sig)
+        signature.add_app_sig(app_sig)
 
-        version = Version.objects.create(signature=pickle.dumps(sig))
+        version = Version.objects.create(signature=signature)
         Evolution.objects.create(version=version,
                                  app_label='evolve_tests',
                                  label='add_new_field')
@@ -826,25 +904,7 @@ class ExtensionManagerTests(SpyAgency, ExtensionTestsMixin, TestCase):
         # We can now enable the extension, which will perform an evolution.
         self.setup_extension(TestExtensionWithApps)
 
-        self.assertFalse(call_command.called_with('evolve'))
-
-    def test_enable_extension_evolve_without_evolutions(self):
-        """Testing ExtensionManager.enable_extension does not evolve database
-        models if extension does not have evolutions
-        """
-        if django_evolution is None:
-            raise nose.SkipTest()
-
-        self.spy_on(call_command)
-
-        class TestExtension(Extension):
-            apps = [
-                'djblets.extensions.tests.apps.model_tests',
-            ]
-
-        self.setup_extension(TestExtension)
-
-        self.assertFalse(call_command.called_with('evolve'))
+        self.assertFalse(Evolver.evolve.called)
 
     def test_disable_unregisters_static_bundles(self):
         """Testing ExtensionManager unregisters static bundles when disabling
