@@ -263,7 +263,7 @@ def sanitize_illegal_chars_for_xml(s):
             ]
 
         ILLEGAL_XML_CHARS_RE = re.compile('[%s]' % ''.join([
-            '%s-%s' % (unichr(low), unichr(high))
+            '%s-%s' % (six.unichr(low), six.unichr(high))
             for low, high in _illegal_unichrs
         ]))
 
