@@ -4,12 +4,12 @@ from __future__ import unicode_literals
 
 from django.contrib.admin.views.decorators import staff_member_required
 from django.http import HttpResponseRedirect
-from django.shortcuts import render
 from django.template.context import RequestContext
 from django.views.decorators.csrf import csrf_protect
 
 from djblets.siteconfig.forms import SiteSettingsForm
 from djblets.siteconfig.models import SiteConfiguration
+from djblets.util.compat.django.shortcuts import render
 
 
 @csrf_protect
