@@ -2,6 +2,7 @@
 
 from __future__ import unicode_literals
 
+import json
 import logging
 import warnings
 
@@ -18,11 +19,13 @@ from django.views.decorators.vary import vary_on_headers
 from djblets.auth.ratelimit import (RATE_LIMIT_API_ANONYMOUS,
                                     RATE_LIMIT_API_AUTHENTICATED,
                                     get_usage_count)
-from djblets.deprecation import RemovedInDjblets20Warning
-from djblets.util.http import (get_modified_since, encode_etag,
+from djblets.util.http import (build_not_modified_from_response,
+                               encode_etag,
                                etag_if_none_match,
-                               set_last_modified, set_etag,
-                               get_http_requested_mimetype)
+                               get_http_requested_mimetype,
+                               get_modified_since,
+                               set_etag,
+                               set_last_modified)
 from djblets.urls.patterns import never_cache_patterns
 from djblets.webapi.auth.backends import check_login
 from djblets.webapi.resources.registry import (get_resource_for_object,
@@ -41,6 +44,7 @@ from djblets.webapi.errors import (DOES_NOT_EXIST,
                                    PERMISSION_DENIED,
                                    RATE_LIMIT_EXCEEDED,
                                    WebAPIError)
+from djblets.webapi.fields import IntFieldType
 
 try:
     # Django >= 1.9
@@ -244,9 +248,9 @@ class WebAPIResource(object):
                 request, method, view, api_format=api_format, *args, **kwargs)
 
             if isinstance(result, WebAPIResponse):
-                return result
+                response = result
             elif isinstance(result, WebAPIError):
-                return WebAPIResponseError(
+                response = WebAPIResponseError(
                     request,
                     err=result,
                     api_format=api_format,
@@ -276,7 +280,7 @@ class WebAPIResource(object):
                             headers['Location'] += '?' + extra_querystr
 
                 if isinstance(result[0], WebAPIError):
-                    return WebAPIResponseError(
+                    response = WebAPIResponseError(
                         request,
                         err=result[0],
                         headers=headers,
@@ -286,7 +290,8 @@ class WebAPIResource(object):
                 else:
                     response_args = self.build_response_args(request)
                     headers.update(response_args.pop('headers', {}))
-                    return WebAPIResponse(
+
+                    response = WebAPIResponse(
                         request,
                         status=result[0],
                         obj=result[1],
@@ -297,11 +302,31 @@ class WebAPIResource(object):
                         }, **kwargs),
                         **response_args)
             elif isinstance(result, HttpResponse):
-                return result
+                response = result
             else:
                 raise AssertionError(result)
+
+            if 200 <= response.status_code < 300:
+                # Check if there's no explicit ETag in a GET/HEAD request and
+                # the resource is told to generate missing ETags automatically.
+                #
+                # Starting in Djblets 2.0, we do this after a response payload
+                # is generated instead of up-front, so that we don't serialize
+                # the resource payload twice.
+                if (self.autogenerate_etags and
+                    method in ('GET', 'HEAD') and
+                    'ETag' not in response):
+                    # There's no ETag, so we'll build one based on the payload
+                    # content and check to see if the client already has it.
+                    etag = encode_etag(response.content)
+                    set_etag(response, etag)
+
+                    if self.are_cache_headers_current(request, etag=etag):
+                        response = build_not_modified_from_response(response)
         else:
-            return HttpResponseNotAllowed(self.allowed_methods)
+            response = HttpResponseNotAllowed(self.allowed_methods)
+
+        return response
 
     @property
     def __name__(self):
@@ -510,18 +535,19 @@ class WebAPIResource(object):
 
         if self.are_cache_headers_current(request, last_modified_timestamp,
                                           etag):
-            return HttpResponseNotModified()
+            response = HttpResponseNotModified()
+        else:
+            data = {
+                self.item_result_key: self.serialize_object(obj,
+                                                            request=request,
+                                                            *args, **kwargs),
+            }
 
-        data = {
-            self.item_result_key: self.serialize_object(obj, request=request,
-                                                        *args, **kwargs),
-        }
-
-        response = WebAPIResponse(request,
-                                  status=200,
-                                  obj=data,
-                                  api_format=api_format,
-                                  **self.build_response_args(request))
+            response = WebAPIResponse(request,
+                                      status=200,
+                                      obj=data,
+                                      api_format=api_format,
+                                      **self.build_response_args(request))
 
         if last_modified_timestamp:
             set_last_modified(response, last_modified_timestamp)
@@ -535,14 +561,14 @@ class WebAPIResource(object):
     @webapi_request_fields(
         optional={
             'start': {
-                'type': int,
+                'type': IntFieldType,
                 'description': 'The 0-based index of the first result in '
                                'the list. The start index is usually the '
                                'previous start index plus the number of '
                                'previous results. By default, this is 0.',
             },
             'max-results': {
-                'type': int,
+                'type': IntFieldType,
                 'description': 'The maximum number of results to return in '
                                'this list. By default, this is 25. There is '
                                'a hard limit of 200; if you need more than '
@@ -656,7 +682,6 @@ class WebAPIResource(object):
         return them in the ``urls.py`` files.
         """
         urlpatterns = never_cache_patterns(
-            '',
             url(r'^$', self, name=self._build_named_url(self.name_plural)),
         )
 
@@ -676,7 +701,6 @@ class WebAPIResource(object):
                 base_regex = r'^'
 
             urlpatterns += never_cache_patterns(
-                '',
                 url(base_regex + r'$', self,
                     name=self._build_named_url(self.name))
             )
@@ -782,6 +806,7 @@ class WebAPIResource(object):
 
         data = {}
         links = {}
+        expand_info = {}
 
         if only_links != []:
             links = self.get_links(self.item_child_resources, obj,
@@ -794,6 +819,12 @@ class WebAPIResource(object):
             expanded_resources = expand.split(',')
             request._djblets_webapi_expanded_resources = expanded_resources
 
+        if expanded_resources:
+            requested_mimetype = get_http_requested_mimetype(
+                request, WebAPIResponse.supported_mimetypes)
+        else:
+            requested_mimetype = None
+
         # Make a copy of the list of expanded resources. We'll be temporarily
         # removing items as we recurse down into any nested objects, to
         # prevent infinite loops. We'll want to make sure we don't
@@ -855,6 +886,13 @@ class WebAPIResource(object):
 
                     if objects:
                         resource = self.get_serializer_for_object(objects[0])
+
+                        expand_info[field] = {
+                            'item_mimetype': resource._build_resource_mimetype(
+                                mimetype=requested_mimetype,
+                                is_list=False),
+                        }
+
                         data[field] = [
                             resource.serialize_object(o, *args, **kwargs)
                             for o in objects
@@ -863,6 +901,13 @@ class WebAPIResource(object):
                         data[field] = []
                 elif isinstance(value, models.Model):
                     resource = self.get_serializer_for_object(value)
+
+                    expand_info[field] = {
+                        'item_mimetype': resource._build_resource_mimetype(
+                            mimetype=requested_mimetype,
+                            is_list=False),
+                    }
+
                     data[field] = resource.serialize_object(
                         value, *args, **kwargs)
                 else:
@@ -885,8 +930,6 @@ class WebAPIResource(object):
             if not found or not resource.model:
                 continue
 
-            del links[resource_name]
-
             extra_kwargs = {
                 self.uri_object_key: getattr(obj, self.model_object_key),
             }
@@ -899,6 +942,18 @@ class WebAPIResource(object):
                     is_list=True, *args, **extra_kwargs)
             ]
 
+            expand_info[resource_name] = {
+                'list_url': links[resource_name]['href'],
+                'list_mimetype': resource._build_resource_mimetype(
+                    mimetype=requested_mimetype,
+                    is_list=True),
+                'item_mimetype': resource._build_resource_mimetype(
+                    mimetype=requested_mimetype,
+                    is_list=False),
+            }
+
+            del links[resource_name]
+
         if only_links is None:
             data['links'] = links
         elif only_links != []:
@@ -908,6 +963,9 @@ class WebAPIResource(object):
                 if link_name in only_links
             ])
 
+        if expand_info:
+            data['_expanded'] = expand_info
+
         # Now that we're done serializing, restore the list of expanded
         # resource for the next call.
         request._djblets_webapi_expanded_resources = orig_expanded_resources
@@ -1158,17 +1216,11 @@ class WebAPIResource(object):
         ``encode_etag`` before returning a value.
         """
         if self.etag_field:
-            etag = six.text_type(getattr(obj, self.etag_field))
-        elif self.autogenerate_etags:
-            etag = self.generate_etag(obj, self.fields, request=request,
-                                      encode_etag=False, **kwargs)
-        else:
-            etag = None
+            return self.encode_etag(
+                request,
+                six.text_type(getattr(obj, self.etag_field)))
 
-        if etag:
-            etag = self.encode_etag(request, etag)
-
-        return etag
+        return None
 
     def encode_etag(self, request, etag, *args, **kwargs):
         """Encodes an ETag for usage in a header.
@@ -1179,30 +1231,45 @@ class WebAPIResource(object):
         return encode_etag('%s:%s' % (request.user.username, etag))
 
     def generate_etag(self, obj, fields, request, encode_etag=True, **kwargs):
-        """Generates an ETag from the serialized values of all given fields.
+        """Generate an ETag from the serialized values of all given fields.
 
-        When called by legacy code, the resulting ETag will be encoded.
-        All consumers are expected to update their get_etag() methods to
-        call encode_etag() directly, and to pass encode_etag=False to this
-        function.
+        .. deprecated:: 2.0
 
-        In a future version, the encode_etag parameter will go away, and
-        this function's behavior will change to not return encoded ETags.
+           This method is no longer called internally, as the auto-generation
+           logic for ETags has changed. It's left here for any resources that
+           may still call it, and is scheduled to be removed in a future
+           release.
+
+        Args:
+            obj (object):
+                The object being serialized.
+
+            fields (list of unicode, unused):
+                An unused list of field names. You can provide any value here.
+                It will be ignored.
+
+            request (django.http.HttpRequest):
+                The HTTP request from the client.
+
+            encode_etag (bool, optional):
+                Whether to encode the ETag to a SHA.
+
+            **kwargs (dict):
+                Extra keyword arguments for :py:meth:`serialize_object`.
+
+        Returns:
+            unicode:
+            The generated ETag.
         """
+        warnings.warn(
+            'WebAPIResource.generate_etag is deprecated and no longer called '
+            'by default. Please provide your own ETag generation in '
+            'get_etag().',
+            DeprecationWarning)
+
         etag = repr(self.serialize_object(obj, request=request, **kwargs))
 
-        # In Djblets 0.8.15, the responsibility for encoding moved to
-        # get_etag(). However, legacy callers may end up calling
-        # generate_etag, expecting the result to be encoded. In this case,
-        # we want to perform the encoding and warn about deprecation.
-        #
-        # Future versions of Djblets will remove the encode_etag argument.
         if encode_etag:
-            warnings.warn('WebAPIResource.generate_etag will stop generating '
-                          'encoded ETags in 2.0.x. Update your get_etag() '
-                          'method to pass encode_etag=False to this function '
-                          'and to call encode_etag() on the result instead.',
-                          RemovedInDjblets20Warning)
             etag = self.encode_etag(request, etag)
 
         return etag
