@@ -14,8 +14,8 @@ import warnings
 
 from django.test.client import BOUNDARY, MULTIPART_CONTENT, encode_multipart
 from django.utils import six
+from django.utils.encoding import force_text
 
-from djblets.deprecation import RemovedInDjblets20Warning
 from djblets.testing.testcases import TestCase
 
 
@@ -324,7 +324,7 @@ class WebAPITestCaseMixin(TestCase):
                 The HTTP response from the API.
         """
         self.assertEquals(response.status_code, 304)
-        self.assertEquals(response.content, '')
+        self.assertEquals(response.content, b'')
 
     def api_call(self, client_http_method, path, data=None,
                  follow_redirects=False, expected_status=200,
@@ -401,15 +401,6 @@ class WebAPITestCaseMixin(TestCase):
         if path.startswith(self.base_url):
             path = path[len(self.base_url):]
 
-        # The `query` argument is outdated. If passed, move its value over to
-        # `data` and raise a deprecation warning.
-        if 'query' in extra:
-            warnings.warn('%s should be passed a `data` argument. `query` '
-                          'is deprecated and will be removed in Djblets 2.0.',
-                          RemovedInDjblets20Warning)
-
-            data = extra.pop('query')
-
         # If the caller is explicitly requested multipart content, ensure we've
         # encoded the data.
         if extra.get('content_type') == MULTIPART_CONTENT:
@@ -435,11 +426,11 @@ class WebAPITestCaseMixin(TestCase):
         self.assertEqual(response.status_code, expected_status)
 
         if expected_status in (204, 405):
-            self.assertEqual(response.content, '')
+            self.assertEqual(response.content, b'')
             rsp = None
         else:
             if expected_status != 302 and expected_json:
-                rsp = json.loads(response.content)
+                rsp = json.loads(force_text(response.content))
             else:
                 rsp = response.content
 
